@@ -404,7 +404,7 @@ regular_dir:
 				continue;
 			}
 
-			data.name = name;
+			data.name = fz::to_wstring(name);
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
@@ -884,8 +884,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	if (!root.empty()) {
 		recursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		recursiveOperation->StartRecursiveOperation(queue_only ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
-			filter.GetActiveFilters(true));
+		recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 	}
 }
 
@@ -1724,9 +1723,10 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		bool program_exists = false;
 		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
 		if (cmd.empty()) {
-			int pos = data.name.Find('.') == -1;
-			if (pos == -1 || (pos == 0 && data.name.Mid(1).Find('.') == -1))
+			auto pos = data.name.find('.');
+			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
 				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
+			}
 		}
 		if (cmd.empty()) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
@@ -1781,7 +1781,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		unsigned int item, index;
 		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
-			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" ")) {
+			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.Len() + 1) == drive + _T(" ")) {
 				break;
 			}
 		}
