@@ -89,10 +89,28 @@ class CDesktopNotification;
 class wxNotificationMessage;
 #endif
 
+class CActionAfterBlocker final
+{
+public:
+	CActionAfterBlocker(CQueueView& queueView)
+		: queueView_(queueView)
+	{
+	}
+
+	~CActionAfterBlocker();
+
+private:
+	friend class CQueueView;
+	CQueueView& queueView_;
+	bool trigger_{};
+};
+
 class CQueueView : public CQueueViewBase, public COptionChangeEventHandler, private EngineNotificationHandler
 {
 	friend class CQueueViewDropTarget;
 	friend class CQueueViewFailed;
+	friend class CActionAfterBlocker;
+
 public:
 	CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue);
 	virtual ~CQueueView();
@@ -141,6 +159,8 @@ public:
 	// Unit is byte/s.
 	wxFileOffset GetCurrentUploadSpeed();
 
+	std::shared_ptr<CActionAfterBlocker> GetActionAfterBlocker();
+
 protected:
 
 #ifdef __WXMSW__
@@ -275,6 +295,8 @@ protected:
 
 	void OnAskPassword();
 
+	std::weak_ptr<CActionAfterBlocker> m_actionAfterBlocker;
+
 	DECLARE_EVENT_TABLE()
 	void OnChar(wxKeyEvent& event);
 
