@@ -5,10 +5,11 @@
 #include "listctrlex.h"
 #include "filelistctrl.h"
 
-class CQueueView;
 class CChmodDialog;
 class CInfoText;
+class CQueueView;
 class CRemoteListViewDropTarget;
+class CWindowTinter;
 
 class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
@@ -99,6 +100,8 @@ protected:
 
 	CServerPath MenuMkdir();
 
+	std::unique_ptr<CWindowTinter> m_windowTinter;
+
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
