@@ -9,13 +9,14 @@ class CFilterControls
 {
 public:
 	CFilterControls();
-	
+
 	void Reset();
 
 	wxChoice* pType;
 	wxChoice* pCondition;
 	wxTextCtrl* pValue;
 	wxChoice* pSet;
+	wxStaticText* pLabel;
 	wxButton* pRemove;
 };
 
@@ -37,8 +38,8 @@ public:
 private:
 	void CalcMinListWidth();
 
-	enum t_filterType GetTypeFromTypeSelection(int selection);
-	void SetSelectionFromType(wxChoice* pChoice, enum t_filterType);
+	t_filterType GetTypeFromTypeSelection(int selection);
+	void SetSelectionFromType(wxChoice* pChoice, t_filterType);
 
 	void MakeControls(const CFilterCondition& condition, int i = -1);
 	void DestroyControls();
@@ -61,6 +62,7 @@ private:
 
 	wxButton* m_pAdd;
 	wxSize m_button_size;
+	wxSize m_size_label_size;
 
 	void OnMore();
 	void OnRemove(int item);
@@ -68,7 +70,7 @@ private:
 
 	void OnListSize(wxSizeEvent& event);
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnButton(wxCommandEvent& event);
 	void OnFilterTypeChange(wxCommandEvent& event);
 	void OnConditionSelectionChange(wxCommandEvent& event);
