@@ -1,28 +1,39 @@
-#include "FileZilla.h"
+#include <filezilla.h>
+
+#include <libfilezilla/event_loop.hpp>
+#include <libfilezilla/iputils.hpp>
+#include <libfilezilla/util.hpp>
+#include "engine_context.h"
 #include "netconfwizard.h"
 #include "Options.h"
 #include "dialogex.h"
 #include "filezillaapp.h"
 #include "externalipresolver.h"
 
+DECLARE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS, -1)
+DEFINE_EVENT_TYPE(fzEVT_ON_EXTERNAL_IP_ADDRESS)
+
 BEGIN_EVENT_TABLE(CNetConfWizard, wxWizard)
 EVT_WIZARD_PAGE_CHANGING(wxID_ANY, CNetConfWizard::OnPageChanging)
 EVT_WIZARD_PAGE_CHANGED(wxID_ANY, CNetConfWizard::OnPageChanged)
-EVT_SOCKET(wxID_ANY, CNetConfWizard::OnSocketEvent)
-EVT_FZ_EXTERNALIPRESOLVE(wxID_ANY, CNetConfWizard::OnExternalIPAddress)
 EVT_BUTTON(XRCID("ID_RESTART"), CNetConfWizard::OnRestart)
 EVT_WIZARD_FINISHED(wxID_ANY, CNetConfWizard::OnFinish)
 EVT_TIMER(wxID_ANY, CNetConfWizard::OnTimer)
+EVT_COMMAND(wxID_ANY, fzEVT_ON_EXTERNAL_IP_ADDRESS, CNetConfWizard::OnExternalIPAddress2)
 END_EVENT_TABLE()
 
-CNetConfWizard::CNetConfWizard(wxWindow* parent, COptions* pOptions)
-: m_parent(parent), m_pOptions(pOptions), m_pSocketServer(0)
+// Mark some strings used by wx as translatable
+#if 0
+TRANSLATE_T("&Next >");
+TRANSLATE_T("< &Back");
+#endif
+
+CNetConfWizard::CNetConfWizard(wxWindow* parent, COptions* pOptions, CFileZillaEngineContext & engine_context)
+	: fz::event_handler(engine_context.GetEventLoop())
+	, m_parent(parent), m_pOptions(pOptions), m_pSocketServer(0)
 {
-	m_socket = 0;
 	m_pIPResolver = 0;
 	m_pSendBuffer = 0;
-	m_pSocketServer = 0;
-	m_pDataSocket = 0;
 
 	m_timer.SetOwner(this);
 
@@ -31,10 +42,13 @@ CNetConfWizard::CNetConfWizard(wxWindow* parent, COptions* pOptions)
 
 CNetConfWizard::~CNetConfWizard()
 {
+	remove_handler();
+
 	delete m_socket;
 	delete m_pIPResolver;
 	delete [] m_pSendBuffer;
 	delete m_pSocketServer;
+	delete m_pDataSocket;
 }
 
 bool CNetConfWizard::Load()
@@ -44,8 +58,7 @@ bool CNetConfWizard::Load()
 
 	wxSize minPageSize = GetPageAreaSizer()->GetMinSize();
 
-	for (int i = 1; i <= 7; i++)
-	{
+	for (int i = 1; i <= 7; ++i) {
 		wxWizardPageSimple* page = new wxWizardPageSimple();
 		bool res = wxXmlResource::Get()->LoadPanel(page, this, wxString::Format(_T("NETCONF_PANEL%d"), i));
 		if (!res)
@@ -130,46 +143,38 @@ bool CNetConfWizard::Run()
 
 void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 {
-	if (event.GetPage() == m_pages[3])
-	{
+	if (event.GetPage() == m_pages[3]) {
 		int mode = XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue() ? 0 : (XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
-		if (mode == 1)
-		{
+		if (mode == 1) {
 			wxTextCtrl* control = XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl);
 			wxString ip = control->GetValue();
-			if (ip == _T(""))
-			{
-				wxMessageBox(_("Please enter your external IP address"));
+			if (ip.empty()) {
+				wxMessageBoxEx(_("Please enter your external IP address"));
 				control->SetFocus();
 				event.Veto();
 				return;
 			}
-			if (!IsIpAddress(ip))
-			{
-				wxMessageBox(_("You have to enter a valid IP address."));
+			if (fz::get_address_type(ip.ToStdWstring()) != fz::address_type::ipv4) {
+				wxMessageBoxEx(_("You have to enter a valid IPv4 address."));
 				control->SetFocus();
 				event.Veto();
 				return;
 			}
 		}
-		else if (mode == 2)
-		{
+		else if (mode == 2) {
 			wxTextCtrl* pResolver = XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl);
 			wxString address = pResolver->GetValue();
-			if (address == _T(""))
-			{
-				wxMessageBox(_("Please enter an URL where to get your external address from"));
+			if (address.empty()) {
+				wxMessageBoxEx(_("Please enter an URL where to get your external address from"));
 				pResolver->SetFocus();
 				event.Veto();
 				return;
 			}
 		}
 	}
-	else if (event.GetPage() == m_pages[4])
-	{
+	else if (event.GetPage() == m_pages[4]) {
 		int mode = XRCCTRL(*this, "ID_ACTIVE_PORTMODE1", wxRadioButton)->GetValue() ? 0 : 1;
-		if (mode)
-		{
+		if (mode) {
 			wxTextCtrl* pPortMin = XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl);
 			wxTextCtrl* pPortMax = XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl);
 			wxString portMin = pPortMin->GetValue();
@@ -179,109 +184,122 @@ void CNetConfWizard::OnPageChanging(wxWizardEvent& event)
 			if (!portMin.ToLong(&min) || !portMax.ToLong(&max) ||
 				min < 1024 || max > 65535 || min > max)
 			{
-				wxMessageBox(_("Please enter a valid portrange."));
+				wxMessageBoxEx(_("Please enter a valid portrange."));
 				pPortMin->SetFocus();
 				event.Veto();
 				return;
 			}
 		}
 	}
-	else if (event.GetPage() == m_pages[5] && !event.GetDirection())
-	{
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->SetLabel(m_nextLabelText);
+	else if (event.GetPage() == m_pages[5] && !event.GetDirection()) {
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->SetLabel(m_nextLabelText);
+		}
 	}
-	else if (event.GetPage() == m_pages[5] && event.GetDirection())
-	{
+	else if (event.GetPage() == m_pages[5] && event.GetDirection()) {
 		if (m_testDidRun)
 			return;
 
 		m_testDidRun = true;
 
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->Disable();
-		wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
-		pPrev->Disable();
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->Disable();
+		}
+		auto pPrev = dynamic_cast<wxButton*>(FindWindow(wxID_BACKWARD));
+		if (pPrev) {
+			pPrev->Disable();
+		}
 		event.Veto();
 
 		PrintMessage(wxString::Format(_("Connecting to %s"), _T("probe.filezilla-project.org")), 0);
-		m_socket = new wxSocketClient(wxSOCKET_NOWAIT);
-		m_socket->SetEventHandler(*this, 0);
-		m_socket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_OUTPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG);
-		m_socket->Notify(true);
+		m_socket = new CSocket(this);
 		m_recvBufferPos = 0;
 
-		wxIPV4address addr;
-		addr.Hostname(_T("probe.filezilla-project.org"));
-		addr.Service(21);
-
-		m_socket->Connect(addr, false);
+		int res = m_socket->Connect(fzT("probe.filezilla-project.org"), 21);
+		if (res && res != EINPROGRESS) {
+			PrintMessage(wxString::Format(_("Connect failed: %s"), CSocket::GetErrorDescription(res)), 1);
+			CloseSocket();
+		}
 	}
 }
 
 void CNetConfWizard::OnPageChanged(wxWizardEvent& event)
 {
-	if (event.GetPage() == m_pages[5])
-	{
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		m_nextLabelText = pNext->GetLabel();
-		pNext->SetLabel(_("&Test"));
+	if (event.GetPage() == m_pages[5]) {
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			m_nextLabelText = pNext->GetLabel();
+			pNext->SetLabel(_("&Test"));
+		}
 	}
-	else if (event.GetPage() == m_pages[6])
-	{
-		wxButton* pPrev = wxDynamicCast(FindWindow(wxID_BACKWARD), wxButton);
-		pPrev->Disable();
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->SetFocus();
+	else if (event.GetPage() == m_pages[6]) {
+		auto pPrev = dynamic_cast<wxButton*>(FindWindow(wxID_BACKWARD));
+		if (pPrev) {
+			pPrev->Disable();
+		}
+		auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+		if (pNext) {
+			pNext->SetFocus();
+		}
 	}
 }
 
-void CNetConfWizard::OnSocketEvent(wxSocketEvent& event)
+void CNetConfWizard::DoOnSocketEvent(CSocketEventSource* s, SocketEventType t, int error)
 {
-	if (!m_socket)
-		return;
-
-	if (event.GetSocket() == m_socket)
-	{
-		switch (event.GetSocketEvent())
+	if (s == m_socket) {
+		if (error) {
+			OnClose();
+			return;
+		}
+		switch (t)
 		{
-		case wxSOCKET_INPUT:
+		case SocketEventType::read:
 			OnReceive();
 			break;
-		case wxSOCKET_OUTPUT:
+		case SocketEventType::write:
 			OnSend();
 			break;
-		case wxSOCKET_LOST:
+		case SocketEventType::close:
 			OnClose();
 			break;
-		case wxSOCKET_CONNECTION:
+		case SocketEventType::connection:
 			OnConnect();
 			break;
+		default:
+			break;
 		}
 	}
-	else if (event.GetSocket() == m_pSocketServer)
-	{
-		switch (event.GetSocketEvent())
-		{
-		case wxSOCKET_LOST:
+	else if (s == m_pSocketServer) {
+		if (error) {
+			PrintMessage(_("Listen socket closed"), 1);
+			CloseSocket();
+			return;
+		}
+		switch (t) {
+		case SocketEventType::close:
 			PrintMessage(_("Listen socket closed"), 1);
 			CloseSocket();
 			break;
-		case wxSOCKET_CONNECTION:
+		case SocketEventType::connection:
 			OnAccept();
 			break;
 		default:
 			break;
 		}
 	}
-	else if (event.GetSocket() == m_pDataSocket)
-	{
-		switch (event.GetSocketEvent())
+	else if (s == m_pDataSocket) {
+		if (error) {
+			OnDataClose();
+			return;
+		}
+		switch (t)
 		{
-		case wxSOCKET_LOST:
+		case SocketEventType::close:
 			OnDataClose();
 			break;
-		case wxSOCKET_INPUT:
+		case SocketEventType::read:
 			OnDataReceive();
 			break;
 		default:
@@ -290,6 +308,7 @@ void CNetConfWizard::OnSocketEvent(wxSocketEvent& event)
 	}
 }
 
+
 void CNetConfWizard::OnSend()
 {
 	if (!m_pSendBuffer)
@@ -298,20 +317,17 @@ void CNetConfWizard::OnSend()
 	if (!m_socket)
 		return;
 
-	int len = strlen(m_pSendBuffer);
-	m_socket->Write(m_pSendBuffer, len);
-	if (m_socket->Error())
-	{
-		if (m_socket->LastError() != wxSOCKET_WOULDBLOCK)
-		{
+	int error;
+	int const len = strlen(m_pSendBuffer);
+	int const written = m_socket->Write(m_pSendBuffer, len, error);
+	if (written < 0) {
+		if (error != EAGAIN) {
 			PrintMessage(_("Failed to send command."), 1);
 			CloseSocket();
 		}
 		return;
 	}
-	int written = m_socket->LastCount();
-	if (written == len)
-	{
+	if (written == len) {
 		delete [] m_pSendBuffer;
 		m_pSendBuffer = 0;
 	}
@@ -326,62 +342,74 @@ void CNetConfWizard::OnClose()
 
 void CNetConfWizard::OnConnect()
 {
+	PrintMessage(_("Connection established, waiting for welcome message."), 0);
 	m_connectSuccessful = true;
 }
 
 void CNetConfWizard::OnReceive()
 {
-	m_socket->Read(m_recvBuffer + m_recvBufferPos, NETCONFBUFFERSIZE - m_recvBufferPos);
-	unsigned int len;
-	if (m_socket->Error() || !(len = m_socket->LastCount()))
-	{
-		PrintMessage(_("Connection lost"), 1);
-		CloseSocket();
-		return;
-	}
+	while (true) {
+		int error;
+		int const read = m_socket->Read(m_recvBuffer + m_recvBufferPos, NETCONFBUFFERSIZE - m_recvBufferPos, error);
+		if (read < 0) {
+			if (error != EAGAIN) {
+				PrintMessage(_("Could not receive data from server."), 1);
+				CloseSocket();
+			}
+			return;
+		}
+		if (!read) {
+			PrintMessage(_("Connection lost"), 1);
+			CloseSocket();
+			return;
+		}
 
-	m_recvBufferPos += len;
+		m_recvBufferPos += read;
 
-	if (m_recvBufferPos < 3)
-		return;
+		if (m_recvBufferPos < 3)
+			return;
 
-	for (int i = 0; i < m_recvBufferPos - 1; i++)
-	{
-		if (m_recvBuffer[i] == '\n')
-		{
+		for (int i = 0; i < m_recvBufferPos - 1; ++i) {
+			if (m_recvBuffer[i] == '\n') {
+				m_testResult = servererror;
+				PrintMessage(_("Invalid data received"), 1);
+				CloseSocket();
+				return;
+			}
+			if (m_recvBuffer[i] != '\r')
+				continue;
+
+			if (m_recvBuffer[i + 1] != '\n') {
+				m_testResult = servererror;
+				PrintMessage(_("Invalid data received"), 1);
+				CloseSocket();
+				return;
+			}
+			m_recvBuffer[i] = 0;
+
+			if (!*m_recvBuffer) {
+				m_testResult = servererror;
+				PrintMessage(_("Invalid data received"), 1);
+				CloseSocket();
+				return;
+			}
+
+			ParseResponse(m_recvBuffer);
+
+			if (!m_socket)
+				return;
+
+			memmove(m_recvBuffer, m_recvBuffer + i + 2, m_recvBufferPos - i - 2);
+			m_recvBufferPos -= i + 2;
+			i = -1;
+		}
+
+		if (m_recvBufferPos == 200) {
 			m_testResult = servererror;
 			PrintMessage(_("Invalid data received"), 1);
 			CloseSocket();
 			return;
 		}
-		if (m_recvBuffer[i] != '\r')
-			continue;
-		
-		if (m_recvBuffer[i + 1] != '\n')
-		{
-			m_testResult = servererror;
-			PrintMessage(_("Invalid data received"), 1);
-			CloseSocket();
-			return;
-		}
-		m_recvBuffer[i] = 0;
-
-		if (!*m_recvBuffer)
-		{
-			m_testResult = servererror;
-			PrintMessage(_("Invalid data received"), 1);
-			CloseSocket();
-			return;
-		}
-
-		ParseResponse(m_recvBuffer);
-
-		if (!m_socket)
-			return;
-
-		memmove(m_recvBuffer, m_recvBuffer + i + 2, m_recvBufferPos - i - 2);
-		m_recvBufferPos -= i + 2;
-		i = -1;
 	}
 }
 
@@ -397,15 +425,13 @@ void CNetConfWizard::ParseResponse(const char* line)
 	str += msg;
 	PrintMessage(str, 3);
 
-	if (len < 3)
-	{
+	if (len < 3) {
 		m_testResult = servererror;
 		PrintMessage(_("Server sent unexpected reply."), 1);
 		CloseSocket();
 		return;
 	}
-	if (line[3] && line[3] != ' ')
-	{
+	if (line[3] && line[3] != ' ') {
 		m_testResult = servererror;
 		PrintMessage(_("Server sent unexpected reply."), 1);
 		CloseSocket();
@@ -421,30 +447,26 @@ void CNetConfWizard::ParseResponse(const char* line)
 		if (line[0] == '2')
 			break;
 
-		if (line[1] == '0' && line[2] == '1')
-		{
+		if (line[1] == '0' && line[2] == '1') {
 			PrintMessage(_("Communication tainted by router or firewall"), 1);
 			m_testResult = tainted;
 			CloseSocket();
 			return;
 		}
-		else if (line[1] == '1' && line[2] == '0')
-		{
+		else if (line[1] == '1' && line[2] == '0') {
 			PrintMessage(_("Wrong external IP address"), 1);
 			m_testResult = mismatch;
 			CloseSocket();
 			return;
 		}
-		else if (line[1] == '1' && line[2] == '1')
-		{
+		else if (line[1] == '1' && line[2] == '1') {
 			PrintMessage(_("Wrong external IP address"), 1);
 			PrintMessage(_("Communication tainted by router or firewall"), 1);
 			m_testResult = mismatchandtainted;
 			CloseSocket();
 			return;
 		}
-		else
-		{
+		else {
 			m_testResult = servererror;
 			PrintMessage(_("Server sent unexpected reply."), 1);
 			CloseSocket();
@@ -482,8 +504,7 @@ void CNetConfWizard::ParseResponse(const char* line)
 			break;
 
 
-		if (line[0] == '5' && line[1] == '0' && (line[2] == '1' || line[2] == '2'))
-		{
+		if (line[0] == '5' && line[1] == '0' && (line[2] == '1' || line[2] == '2')) {
 			m_testResult = tainted;
 			PrintMessage(_("PORT command tainted by router or firewall."), 1);
 			CloseSocket();
@@ -524,13 +545,13 @@ void CNetConfWizard::ParseResponse(const char* line)
 		}
 		break;
 	}
-	
+
 	m_state++;
 
 	SendNextCommand();
 }
 
-void CNetConfWizard::PrintMessage(const wxString& msg, int type)
+void CNetConfWizard::PrintMessage(const wxString& msg, int)
 {
 	XRCCTRL(*this, "ID_RESULTS", wxTextCtrl)->AppendText(msg + _T("\n"));
 }
@@ -542,15 +563,17 @@ void CNetConfWizard::CloseSocket()
 
 	PrintMessage(_("Connection closed"), 0);
 
-	wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-	pNext->Enable();
-	pNext->SetLabel(m_nextLabelText);
+	auto pNext = dynamic_cast<wxButton*>(FindWindow(wxID_FORWARD));
+	if (pNext) {
+		pNext->Enable();
+		pNext->SetLabel(m_nextLabelText);
+	}
 
 	wxString text[5];
 	if (!m_connectSuccessful)
 	{
 		text[0] = _("Connection with the test server failed.");
-		text[1] = _("Please check on http://filezilla-project.org/probe.php that the server is running and carefully check your settings again.");
+		text[1] = _("Please check on https://filezilla-project.org/probe.php that the server is running and carefully check your settings again.");
 		text[2] = _("If the problem persists, some router and/or firewall keeps blocking FileZilla.");
 	}
 	else
@@ -561,9 +584,10 @@ void CNetConfWizard::CloseSocket()
 			text[0] = _("Connection with server got closed prematurely.");
 			text[1] = _("Please ensure you have a stable internet connection and carefully check your settings again.");
 			text[2] = _("If the problem persists, some router and/or firewall keeps interrupting the connection.");
-			text[3] = _("See also: http://wiki.filezilla-project.org/Network_Configuration");
+			text[3] = _("See also: https://wiki.filezilla-project.org/Network_Configuration");
 			break;
 		case successful:
+			PrintMessage(_("Test finished successfully"), 0);
 			text[0] = _("Congratulations, your configuration seems to be working.");
 			text[1] = _("You should have no problems connecting to other servers, file transfers should work properly.");
 			text[2] = _("If you keep having problems with a specific server, the server itself or a remote router or firewall might be misconfigured. In this case try to toggle passive mode and contact the server administrator for help.");
@@ -574,7 +598,7 @@ void CNetConfWizard::CloseSocket()
 			text[0] = _("The server sent an unexpected or unrecognized reply.");
 			text[1] = _("This means that some router and/or firewall is still interfering with FileZilla.");
 			text[2] = _("Re-run the wizard and carefully check your settings and configure all routers and firewalls accordingly.");
-			text[3] = _("See also: http://wiki.filezilla-project.org/Network_Configuration");
+			text[3] = _("See also: https://wiki.filezilla-project.org/Network_Configuration");
 			break;
 		case tainted:
 			text[0] = _("Active mode FTP test failed. FileZilla knows the correct external IP address, but your router or firewall has misleadingly modified the sent address.");
@@ -615,7 +639,7 @@ void CNetConfWizard::CloseSocket()
 		case externalfailed:
 			text[0] = _("Failed to retrieve the external IP address.");
 			text[1] = _("Please make sure FileZilla is allowed to establish outgoing connections and make sure you typed the address of the address resolver correctly.");
-			text[2] = wxString::Format(_("The address you entered was: %s"), XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue().c_str());
+			text[2] = wxString::Format(_("The address you entered was: %s"), XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue());
 			break;
 		case datatainted:
 			text[0] = _("Transferred data got tainted.");
@@ -625,11 +649,13 @@ void CNetConfWizard::CloseSocket()
 			break;
 		}
 	}
-	for (unsigned int i = 0; i < 5; i++)
-	{
+	for (unsigned int i = 0; i < 5; ++i) {
 		wxString name = wxString::Format(_T("ID_SUMMARY%d"), i + 1);
 		int id = wxXmlResource::GetXRCID(name);
-		wxDynamicCast(FindWindowById(id, this), wxStaticText)->SetLabel(text[i]);
+		auto ctrl = dynamic_cast<wxStaticText*>(FindWindowById(id, this));
+		if (ctrl) {
+			ctrl->SetLabel(text[i]);
+		}
 	}
 	m_pages[6]->GetSizer()->Layout();
 	m_pages[6]->GetSizer()->Fit(m_pages[6]);
@@ -677,64 +703,51 @@ bool CNetConfWizard::Send(wxString cmd)
 
 wxString CNetConfWizard::GetExternalIPAddress()
 {
+	wxString ret;
+
 	wxASSERT(m_socket);
 
 	int mode = XRCCTRL(*this, "ID_ACTIVEMODE1", wxRadioButton)->GetValue() ? 0 : (XRCCTRL(*this, "ID_ACTIVEMODE2", wxRadioButton)->GetValue() ? 1 : 2);
-	if (!mode)
-	{
-		wxIPV4address addr;
-		if (!m_socket->GetLocal(addr))
-		{
+	if (!mode) {
+		ret = m_socket->GetLocalIP();
+		if (ret.empty()) {
 			PrintMessage(_("Failed to retrieve local ip address. Aborting"), 1);
 			CloseSocket();
-			return _T("");
 		}
-
-		return addr.IPAddress();
 	}
-	else if (mode == 1)
-	{
+	else if (mode == 1) {
 		wxTextCtrl* control = XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl);
-		return control->GetValue();
+		ret = control->GetValue();
 	}
-	else if (mode == 2)
-	{
-		if (!m_pIPResolver)
-		{
+	else if (mode == 2) {
+		if (!m_pIPResolver) {
 			wxTextCtrl* pResolver = XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl);
 			wxString address = pResolver->GetValue();
 
-			PrintMessage(wxString::Format(_("Retrieving external IP address from %s"), address.c_str()), 0);
+			PrintMessage(wxString::Format(_("Retrieving external IP address from %s"), address), 0);
 
-			m_pIPResolver = new CExternalIPResolver(this);
-			m_pIPResolver->GetExternalIP(address, true);
+			m_pIPResolver = new CExternalIPResolver(*this);
+			m_pIPResolver->GetExternalIP(address, CSocket::ipv4, true);
 			if (!m_pIPResolver->Done())
-				return _T("");
+				return wxString();
 		}
-		if (!m_pIPResolver->Successful())
-		{
-			delete m_pIPResolver;
-			m_pIPResolver = 0;
-
+		if (m_pIPResolver->Successful()) {
+			ret = m_pIPResolver->GetIP();
+		}
+		else {
 			PrintMessage(_("Failed to retrieve external ip address, aborting"), 1);
 
 			m_testResult = externalfailed;
 			CloseSocket();
-			return _T("");
 		}
-
-		wxString ip = m_pIPResolver->GetIP();
-
 		delete m_pIPResolver;
 		m_pIPResolver = 0;
-
-		return ip;
 	}
 
-	return _T("");
+	return ret;
 }
 
-void CNetConfWizard::OnExternalIPAddress(fzExternalIPResolveEvent& event)
+void CNetConfWizard::OnExternalIPAddress2(wxCommandEvent&)
 {
 	if (!m_pIPResolver)
 		return;
@@ -764,18 +777,23 @@ void CNetConfWizard::SendNextCommand()
 		{
 			PrintMessage(_("Checking for correct external IP address"), 0);
 			wxString ip = GetExternalIPAddress();
-			if (ip == _T(""))
+			if (ip.empty())
 				return;
+			if (!fz::get_ipv6_long_form(ip.ToStdWstring()).empty()) {
+				PrintMessage(_("You appear to be using an IPv6-only host. This wizard does not support this environment."), 1);
+				CloseSocket();
+				return;
+			}
 			m_externalIP = ip;
 
 			wxString hexIP = ip;
-			for (unsigned int i = 0; i < hexIP.Length(); i++)
-			{
-				wxChar& c = hexIP[i];
+			for (unsigned int i = 0; i < hexIP.Length(); i++) {
+				wxChar c = hexIP[i];
 				if (c == '.')
 					c = '-';
 				else
 					c = c - '0' + 'a';
+				hexIP.SetChar(i, c);
 			}
 
 			if (!Send(_T("IP ") + ip + _T(" ") + hexIP))
@@ -798,7 +816,7 @@ void CNetConfWizard::SendNextCommand()
 		}
 	case 5:
 		{
-			wxString cmd = wxString::Format(_T("PORT %s,%d,%d"), m_externalIP.c_str(), m_listenPort / 256, m_listenPort % 256);
+			wxString cmd = wxString::Format(_T("PORT %s,%d,%d"), m_externalIP, m_listenPort / 256, m_listenPort % 256);
 			cmd.Replace(_T("."), _T(","));
 			Send(cmd);
 		}
@@ -816,7 +834,7 @@ void CNetConfWizard::SendNextCommand()
 	}
 }
 
-void CNetConfWizard::OnRestart(wxCommandEvent& event)
+void CNetConfWizard::OnRestart(wxCommandEvent&)
 {
 	ResetTest();
 	ShowPage(m_pages[0], false);
@@ -839,11 +857,10 @@ void CNetConfWizard::ResetTest()
 		XRCCTRL(*this, "ID_RESULTS", wxTextCtrl)->SetLabel(_T(""));
 }
 
-void CNetConfWizard::OnFinish(wxWizardEvent& event)
+void CNetConfWizard::OnFinish(wxWizardEvent&)
 {
-	if (m_testResult != successful)
-	{
-		if (wxMessageBox(_("The test did not succeed. Do you really want to save the settings?"), _("Save settings?"), wxYES_NO | wxICON_QUESTION) != wxYES)
+	if (m_testResult != successful) {
+		if (wxMessageBoxEx(_("The test did not succeed. Do you really want to save the settings?"), _("Save settings?"), wxYES_NO | wxICON_QUESTION) != wxYES)
 			return;
 	}
 
@@ -862,7 +879,7 @@ void CNetConfWizard::OnFinish(wxWizardEvent& event)
 	long tmp;
 	XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_LOW, tmp);
 	XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&tmp); m_pOptions->SetOption(OPTION_LIMITPORTS_HIGH, tmp);
-	
+
 	m_pOptions->SetOption(OPTION_EXTERNALIP, XRCCTRL(*this, "ID_ACTIVEIP", wxTextCtrl)->GetValue());
 	m_pOptions->SetOption(OPTION_EXTERNALIPRESOLVER, XRCCTRL(*this, "ID_ACTIVERESOLVER", wxTextCtrl)->GetValue());
 	m_pOptions->SetOption(OPTION_NOEXTERNALONLOCAL, XRCCTRL(*this, "ID_NOEXTERNALONLOCAL", wxCheckBox)->GetValue());
@@ -883,8 +900,8 @@ int CNetConfWizard::CreateListenSocket()
 		long high;
 		XRCCTRL(*this, "ID_ACTIVE_PORTMIN", wxTextCtrl)->GetValue().ToLong(&low);
 		XRCCTRL(*this, "ID_ACTIVE_PORTMAX", wxTextCtrl)->GetValue().ToLong(&high);
-		
-		int mid = GetRandomNumber(low, high);
+
+		int mid = fz::random_number(low, high);
 		wxASSERT(mid >= low && mid <= high);
 
 		for (int port = mid; port <= high; port++)
@@ -900,67 +917,60 @@ int CNetConfWizard::CreateListenSocket()
 
 int CNetConfWizard::CreateListenSocket(unsigned int port)
 {
-	wxIPV4address addr;
-	if (!addr.AnyAddress())
-		return 0;
+	m_pSocketServer = new CSocket(this);
+	int res = m_pSocketServer->Listen(m_socket ? m_socket->GetAddressFamily() : CSocket::unspec, port);
 
-	if (!addr.Service(port))
-		return 0;
-
-	m_pSocketServer = new wxSocketServer(addr, wxSOCKET_NOWAIT);
-	if (!m_pSocketServer->Ok())
-	{
+	if (res < 0) {
 		delete m_pSocketServer;
 		m_pSocketServer = 0;
 		return 0;
 	}
 
-	m_pSocketServer->SetEventHandler(*this);
-	m_pSocketServer->SetFlags(wxSOCKET_NOWAIT);
-	m_pSocketServer->SetNotify(wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG);
-	m_pSocketServer->Notify(true);
-
-
 	if (port)
 		return port;
 
 	// Get port number from socket
-	if (!m_pSocketServer->GetLocal(addr))
-	{
+	int error;
+	res = m_pSocketServer->GetLocalPort(error);
+	if (res <= 0) {
 		delete m_pSocketServer;
 		m_pSocketServer = 0;
 		return 0;
 	}
-	return addr.Service();
+	return static_cast<unsigned int>(port);
 }
 
 void CNetConfWizard::OnAccept()
 {
+	if (!m_socket || !m_pSocketServer) {
+		return;
+	}
 	if (m_pDataSocket)
 		return;
-	m_pDataSocket = m_pSocketServer->Accept(false);
+
+	int error;
+	m_pDataSocket = m_pSocketServer->Accept(error);
 	if (!m_pDataSocket)
 		return;
+	m_pDataSocket->SetEventHandler(this);
 
-	wxIPV4address peerAddr, dataPeerAddr;
-	if (!m_socket->GetPeer(peerAddr))
-	{
+	std::string peerAddr = m_socket->GetPeerIP();
+	std::string dataPeerAddr = m_pDataSocket->GetPeerIP();
+	if (peerAddr.empty()) {
 		delete m_pDataSocket;
 		m_pDataSocket = 0;
 		PrintMessage(_("Failed to get peer address of control connection, connection closed."), 1);
 		CloseSocket();
 		return;
 	}
-	if (!m_pDataSocket->GetPeer(dataPeerAddr))
-	{
+	if (dataPeerAddr.empty()) {
 		delete m_pDataSocket;
 		m_pDataSocket = 0;
 		PrintMessage(_("Failed to get peer address of data connection, connection closed."), 1);
 		CloseSocket();
 		return;
 	}
-	if (peerAddr.IPAddress() != dataPeerAddr.IPAddress())
-	{
+	if (peerAddr != dataPeerAddr) {
 		delete m_pDataSocket;
 		m_pDataSocket = 0;
 		PrintMessage(_("Warning, ignoring data connection from wrong IP."), 0);
@@ -968,32 +978,31 @@ void CNetConfWizard::OnAccept()
 	}
 	delete m_pSocketServer;
 	m_pSocketServer = 0;
-
-	m_pDataSocket->SetEventHandler(*this);
-	m_pDataSocket->SetFlags(wxSOCKET_NOWAIT);
-	m_pDataSocket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG);
-	m_pDataSocket->Notify(true);
 }
 
 void CNetConfWizard::OnDataReceive()
 {
 	char buffer[100];
-	m_pDataSocket->Read(buffer, 99);
-	unsigned int len;
-	if (m_pDataSocket->Error() || !(len = m_pDataSocket->LastCount()))
-	{
+	int error;
+	int const read = m_pDataSocket->Read(buffer, 99, error);
+	if (!read) {
 		PrintMessage(_("Data socket closed too early."), 1);
 		CloseSocket();
 		return;
 	}
-	buffer[len] = 0;
+	if (read < 0) {
+		if (error != EAGAIN) {
+			PrintMessage(_("Could not read from data socket."), 1);
+			CloseSocket();
+		}
+		return;
+	}
+	buffer[read] = 0;
 
 	int data = 0;
 	const char* p = buffer;
-	while (*p && *p != ' ')
-	{
-		if (*p < '0' || *p > '9')
-		{
+	while (*p && *p != ' ') {
+		if (*p < '0' || *p > '9') {
 			m_testResult = datatainted;
 			PrintMessage(_("Received data tainted"), 1);
 			CloseSocket();
@@ -1001,43 +1010,38 @@ void CNetConfWizard::OnDataReceive()
 		}
 		data = data * 10 + *p++ - '0';
 	}
-	if (data != m_data)
-	{
+	if (data != m_data) {
 		m_testResult = datatainted;
 		PrintMessage(_("Received data tainted"), 1);
 		CloseSocket();
 		return;
 	}
 	p++;
-	if (p - buffer != (int)len - 4)
-	{
+	if (p - buffer != read - 4) {
 		PrintMessage(_("Failed to receive data"), 1);
 		CloseSocket();
 		return;
 	}
 
 	wxUint32 ip = 0;
-	for (const wxChar* q = m_externalIP.c_str(); *q; q++)
-	{
+	for (const wxChar* q = m_externalIP.c_str(); *q; ++q) {
 		if (*q == '.')
 			ip *= 256;
 		else
 			ip = ip - (ip % 256) + (ip % 256) * 10 + *q - '0';
 	}
 	ip = wxUINT32_SWAP_ON_LE(ip);
-	if (memcmp(&ip, p, 4))
-	{
+	if (memcmp(&ip, p, 4)) {
 		m_testResult = datatainted;
 		PrintMessage(_("Received data tainted"), 1);
 		CloseSocket();
 		return;
 	}
-	
+
 	delete m_pDataSocket;
 	m_pDataSocket = 0;
 
-	if (gotListReply)
-	{
+	if (gotListReply) {
 		m_state++;
 		SendNextCommand();
 	}
@@ -1046,8 +1050,7 @@ void CNetConfWizard::OnDataReceive()
 void CNetConfWizard::OnDataClose()
 {
 	OnDataReceive();
-	if (m_pDataSocket)
-	{
+	if (m_pDataSocket) {
 		PrintMessage(_("Data socket closed too early."), 0);
 		CloseSocket();
 		return;
@@ -1055,8 +1058,7 @@ void CNetConfWizard::OnDataClose()
 	delete m_pDataSocket;
 	m_pDataSocket = 0;
 
-	if (gotListReply)
-	{
+	if (gotListReply) {
 		m_state++;
 		SendNextCommand();
 	}
@@ -1070,3 +1072,24 @@ void CNetConfWizard::OnTimer(wxTimerEvent& event)
 	PrintMessage(_("Connection timed out."), 0);
 	CloseSocket();
 }
+
+void CNetConfWizard::operator()(fz::event_base const& ev)
+{
+	fz::dispatch<CSocketEvent, CExternalIPResolveEvent>(ev, this
+		, &CNetConfWizard::OnSocketEvent
+		, &CNetConfWizard::OnExternalIPAddress);
+}
+
+void CNetConfWizard::OnExternalIPAddress()
+{
+	QueueEvent(new wxCommandEvent(fzEVT_ON_EXTERNAL_IP_ADDRESS));
+}
+
+void CNetConfWizard::OnSocketEvent(CSocketEventSource* s, SocketEventType t, int error)
+{
+	if (!s) {
+		return;
+	}
+
+	CallAfter([=]{DoOnSocketEvent(s, t, error);});
+}
