@@ -3,6 +3,8 @@
 
 #include "dialogex.h"
 
+#include <wx/timer.h>
+
 class CWelcomeDialog : public wxDialogEx
 {
 public:
@@ -12,6 +14,9 @@ public:
 	bool Run(wxWindow* parent, bool force = false, bool delay = false);
 
 protected:
+
+	void InitFooter(wxString const& resources);
+
 	wxTimer m_delayedShowTimer;
 
 	DECLARE_EVENT_TABLE()
